@@ -55,6 +55,7 @@
 #include <mapi.h>
 #include <exchange-account.h>
 #include <exchange-hierarchy.h>
+#include <exchange-esource.h>
 #include <e-folder-exchange.h>
 #include "e-book-backend-exchange.h"
 #include "exchange-component.h"
@@ -395,8 +396,10 @@ e_contact_from_props (EBookBackendExchange *be, E2kResult *result)
 			stream_mem = (CamelStreamMem *)stream;
 			camel_data_wrapper_decode_to_stream (content, stream);
 
-			photo.data = stream_mem->buffer->data;
-			photo.length = stream_mem->buffer->len;
+			photo.type = E_CONTACT_PHOTO_TYPE_INLINED;
+			photo.data.inlined.mime_type = NULL;
+			photo.data.inlined.data = stream_mem->buffer->data;
+			photo.data.inlined.length = stream_mem->buffer->len;
 			e_contact_set (contact, E_CONTACT_PHOTO, &photo);
 
 			camel_object_unref (stream);
@@ -859,7 +862,7 @@ populate_date(EContactField field, EContact *new_contact, void *data)
 
 	dt.year = then->tm_year + 1900;
 	dt.month = then->tm_mon + 1;
-	dt.day = then->tm_mday;
+	dt.day = then->tm_mday + 1;
 	
 	e_contact_set (new_contact, field, &dt);
 }
@@ -1194,7 +1197,8 @@ build_message (const char *from_name, const char *from_email,
 
 		/* Determine the MIME type of the photo */
 		loader = gdk_pixbuf_loader_new ();
-		gdk_pixbuf_loader_write (loader, photo->data, photo->length, NULL);
+		gdk_pixbuf_loader_write (loader, photo->data.inlined.data, 
+					 photo->data.inlined.length, NULL);
 		gdk_pixbuf_loader_close (loader, NULL);
 
 		format = gdk_pixbuf_loader_get_format (loader);
@@ -1213,7 +1217,8 @@ build_message (const char *from_name, const char *from_email,
 
 		/* Build the MIME part */
 		photo_ba = g_byte_array_new ();
-		g_byte_array_append (photo_ba, photo->data, photo->length);
+		g_byte_array_append (photo_ba, photo->data.inlined.data, 
+				     photo->data.inlined.length);
 		stream = camel_stream_mem_new_with_byte_array (photo_ba);
 
 		wrapper = camel_data_wrapper_new ();
@@ -1268,7 +1273,7 @@ do_put (EBookBackendExchange *be, EDataBook *book,
 	body = build_message (hier->owner_name, hier->owner_email,
 			      subject, note, photo);
 	status = e2k_context_put (be->priv->ctx, NULL, uri, "message/rfc822",
-				  body->data, body->len, NULL);
+				  (char *) body->data, body->len, NULL);
 	g_byte_array_free (body, TRUE);
 
 	return status;
@@ -1410,7 +1415,7 @@ e_book_backend_exchange_modify_contact (EBookBackendSync  *backend,
 			/* Do the PUT request if we need to. */
 			char *old_note, *new_note;
 			EContactPhoto *old_photo, *new_photo;
-			gboolean changed;
+			gboolean changed = FALSE;
 
 			old_note = e_contact_get (old_contact, E_CONTACT_NOTE);
 			old_photo = e_contact_get (old_contact, E_CONTACT_PHOTO);
@@ -1420,17 +1425,23 @@ e_book_backend_exchange_modify_contact (EBookBackendSync  *backend,
 			if ((old_note && !new_note) ||
 		    	    (new_note && !old_note) ||
 		    	    (old_note && new_note &&
-		     	strcmp (old_note, new_note) != 0))
+		     	     strcmp (old_note, new_note) != 0))
 				changed = TRUE;
-			else if ((old_photo && !new_photo) ||
-				 (new_photo && !old_photo) ||
-			 	(old_photo && new_photo &&
-			  	((old_photo->length != new_photo->length) ||
-			   	(memcmp (old_photo->data, new_photo->data, 
-					 old_photo->length) != 0))))
-					changed = TRUE;
-			else
-				changed = FALSE;
+			else if ((old_photo && !new_photo) || (new_photo && !old_photo))
+				changed = TRUE;
+			else if (old_photo && new_photo) {
+				if ((old_photo->type == new_photo->type) && 
+				     old_photo->type == E_CONTACT_PHOTO_TYPE_INLINED) {
+					changed = ((old_photo->data.inlined.length == new_photo->data.inlined.length)
+                                 	     	    && !memcmp (old_photo->data.inlined.data, 
+							 	new_photo->data.inlined.data, 
+								old_photo->data.inlined.length));
+				}
+				else if ((old_photo->type == new_photo->type) &&
+					  old_photo->type == E_CONTACT_PHOTO_TYPE_URI) {
+					changed = !strcmp (old_photo->data.uri, new_photo->data.uri);
+				}
+			}
 
 			if (changed) {
 				status = do_put (be, book, uri,
@@ -1942,6 +1953,9 @@ e_book_backend_exchange_start_book_view (EBookBackend  *backend,
 						  http_status_to_pas (status));
 		bonobo_object_unref (book_view);
 
+		/* also update the folder list */
+		exchange_account_rescan_tree (bepriv->account);
+
 	default:
 		break;
 	}
@@ -2308,12 +2322,25 @@ e_book_backend_exchange_remove (EBookBackendSync *backend, EDataBook *book, guin
 {
 	EBookBackendExchange *be = E_BOOK_BACKEND_EXCHANGE (backend);
 	EBookBackendExchangePrivate *bepriv = be->priv;
-	ExchangeAccountFolderResult result;
-	const char *uri;
+	ExchangeAccountFolderResult result = EXCHANGE_ACCOUNT_FOLDER_GENERIC_ERROR;
+	const char *int_uri = NULL;
 
 	d(printf("ebbe_remove(%p, %p)\n", backend, book));
-	uri = e_folder_exchange_get_internal_uri (bepriv->folder);
-	result = exchange_account_remove_folder (bepriv->account, uri);
+	int_uri = e_folder_exchange_get_internal_uri (bepriv->folder);
+	if (int_uri)
+		result = exchange_account_remove_folder (bepriv->account, int_uri);
+	else {
+		ExchangeAccount *account;
+
+		/* internal URI is NULL, mostly the case where folder doesn't exists anymore 
+		 * in the server, update the gconf sources. 
+		 */
+		account = exchange_component_get_account_for_uri (global_exchange_component, NULL);
+		if (exchange_account_get_context (account)) {
+			remove_folder_esource (account, EXCHANGE_CONTACTS_FOLDER, bepriv->exchange_uri);
+			result = EXCHANGE_ACCOUNT_FOLDER_OK;
+		}
+	}
 	if (result == EXCHANGE_ACCOUNT_FOLDER_OK)
 		return GNOME_Evolution_Addressbook_Success;
 	else if (result == EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST)
