@@ -1893,7 +1893,7 @@ build_message_from_document (E2kContext *ctx, E2kOperation *op,
 	E2kHTTPStatus status;
 	E2kResult *results;
 	int nresults = 0;
-	GByteArray *message;
+	GString *message;
 	char *headers;
 
 	status = e2k_context_propfind (ctx, op, uri,
@@ -1907,15 +1907,15 @@ build_message_from_document (E2kContext *ctx, E2kOperation *op,
 
 	headers = mail_util_mapi_to_smtp_headers (results[0].props);
 
-	message = g_byte_array_new ();
-	g_byte_array_append (message, headers, strlen (headers));
+	message = g_string_new (headers);
+	g_string_append_len (message, *body, *len);
+
 	g_free (headers);
-	g_byte_array_append (message, *body, *len);
 	g_free (*body);
 
-	*body = (char *)message->data;
 	*len = message->len;
-	g_byte_array_free (message, FALSE);
+	*body = g_string_free (message, FALSE);
+
 	e2k_results_free (results, nresults);
 	return status;
 }
@@ -1926,7 +1926,6 @@ unmangle_delegated_meeting_request (MailStubExchange *mse, E2kOperation *op,
 				    char **body, int *len)
 {
 	const char *prop = PR_RCVD_REPRESENTING_EMAIL_ADDRESS;
-	GByteArray *message;
 	char *delegator_dn, *delegator_uri;
 	ExchangeAccount *account;
 	E2kGlobalCatalog *gc;
@@ -1974,16 +1973,17 @@ unmangle_delegated_meeting_request (MailStubExchange *mse, E2kOperation *op,
 	if (delegator_uri) {
 		folder = exchange_account_get_folder (account, delegator_uri);
 		if (folder) {
-			message = g_byte_array_new ();
-			g_byte_array_append (message, *body, *len);
+			GString *message;
+
+			message = g_string_new_len (*body, *len);
 			mail_util_demangle_delegated_meeting (
 				message, entry->display_name,
 				entry->email,
 				e_folder_get_physical_uri (folder));
 			g_free (*body);
-			*body = (char *)message->data;
+			*body = message->str;
 			*len = message->len;
-			g_byte_array_free (message, FALSE);
+			*body = g_string_free (message, FALSE);
 		}
 		g_free (delegator_uri);
 	}
@@ -2284,34 +2284,46 @@ get_folder_info (MailStub *stub, const char *top, guint32 store_flags)
 	EFolder *folder;
 	const char *type, *name, *uri, *path, *inbox_uri = NULL;
 	int unread_count, i, toplen = top ? strlen (top) : 0;
-	guint32 folder_flags;
+	guint32 folder_flags = 0;
 	gboolean recursive, subscribed, single_folder = FALSE;
+	int mode = -1;
+	char *full_path;
 
 	recursive = (store_flags & CAMEL_STUB_STORE_FOLDER_INFO_RECURSIVE);
 	subscribed = (store_flags & CAMEL_STUB_STORE_FOLDER_INFO_SUBSCRIBED);
 
-	exchange_account_rescan_tree (mse->account);
-
+	exchange_account_is_offline (mse->account, &mode);
 	if (!subscribed) {
-		exchange_account_open_folder (mse->account, "/public");
+		ExchangeAccountResult result = -1;
+		d(g_print ("%s(%d):%s: NOT SUBSCRIBED top = [%s]\n", __FILE__, __LINE__, __GNUC_PRETTY_FUNCTION__, top));
+		if (!toplen)
+			result = exchange_account_open_folder (mse->account, "/public");
+		if (result ==  EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST) {
+			hier = exchange_account_get_hierarchy_by_type (mse->account, EXCHANGE_HIERARCHY_PUBLIC);
+			if (hier)
+				exchange_hierarchy_scan_subtree (hier, hier->toplevel, mode);
+		} else
+			d(g_print ("%s(%d):%s: NOT SUBSCRIBED - open_folder returned = [%d]\n", __FILE__, __LINE__, __GNUC_PRETTY_FUNCTION__, result));
 	}
 
 	if (!recursive && toplen) {
-		char *full_path;
-
+		d(g_print ("%s(%d):%s: NOT RECURSIVE and toplen top = [%s]\n", __FILE__, __LINE__, __GNUC_PRETTY_FUNCTION__, top));
 		full_path = g_strdup_printf ("/%s", top);
-		folder = exchange_account_get_folder (mse->account, full_path);
-		g_free (full_path);
-		if (folder) {
+		folders = exchange_account_get_folder_tree (mse->account, full_path);
+			/*
 			type = e_folder_get_type_string (folder);
 			if ((!strcmp (type, "mail")) || 
 				(!strcmp (type, "mail/public"))) {
+				d(g_print ("%s(%d):%s: Setting single_folder = TRUE\n", __FILE__, __LINE__, __PRETTY_FUNCTION__));
 				folders = g_ptr_array_new ();
 				g_ptr_array_add (folders, folder);
 				single_folder = TRUE;
 			}
-		}
+			*/
+		g_free (full_path);
 	} else {
+		d(g_print ("%s(%d):%s calling exchange_account_get_folders \n", __FILE__, __LINE__,
+					   __GNUC_PRETTY_FUNCTION__));
 		folders = exchange_account_get_folders (mse->account);
 	}
 
@@ -2393,6 +2405,14 @@ return_data:
 			if (inbox_uri && !strcmp (uri, inbox_uri))
 				folder_flags |= CAMEL_STUB_FOLDER_SYSTEM|CAMEL_STUB_FOLDER_TYPE_INBOX;
 
+			if (!e_folder_exchange_get_has_subfolders (folder)) {
+				d(printf ("%s:%d:%s - %s has no subfolders", __FILE__, __LINE__, __GNUC_PRETTY_FUNCTION__,
+					  name));
+				folder_flags |= CAMEL_STUB_FOLDER_NOCHILDREN;
+			}
+
+			d(g_print ("folder flags is : %d\n", folder_flags));
+			
 			g_ptr_array_add (names, (char *)name);
 			g_ptr_array_add (uris, (char *)uri);
 			g_array_append_val (unread, unread_count);
@@ -2696,12 +2716,12 @@ subscribe_folder (MailStub *stub, const char *folder_name)
 static void
 unsubscribe_folder (MailStub *stub, const char *folder_name)
 {
-	d(printf ("unsubscribe folder : %s\n", folder_name));
 	MailStubExchange *mse = MAIL_STUB_EXCHANGE (stub);
 	ExchangeAccountFolderResult result;
 	EFolder *folder;
 	char *path, *pub_name;
 
+	d(printf ("unsubscribe folder : %s\n", folder_name));
 	path = g_build_filename ("/", folder_name, NULL);
 	folder = exchange_account_get_folder (mse->account, path);
 	if (!folder) {
