@@ -119,7 +119,7 @@ class_init (GObjectClass *object_class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ExchangeConfigListenerClass, exchange_account_created),
 			      NULL, NULL,
-			      e2k_marshal_NONE__POINTER,
+			      g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
 	signals[EXCHANGE_ACCOUNT_REMOVED] =
@@ -128,7 +128,7 @@ class_init (GObjectClass *object_class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ExchangeConfigListenerClass, exchange_account_removed),
 			      NULL, NULL,
-			      e2k_marshal_NONE__POINTER,
+			      g_cclosure_marshal_VOID__POINTER,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_POINTER);
 }
@@ -361,10 +361,11 @@ migrate_account_esource (EAccount *account,
 void
 exchange_config_listener_migrate_esources (ExchangeConfigListener *config_listener)
 {
+	EAccount *account;
+
 	g_return_if_fail (config_listener != NULL);
 
-	EAccount *account = config_listener->priv->configured_account;
-
+	account = config_listener->priv->configured_account;
 	migrate_account_esource (account, EXCHANGE_CALENDAR_FOLDER);
 	migrate_account_esource (account, EXCHANGE_TASKS_FOLDER);
 	migrate_account_esource (account, EXCHANGE_CONTACTS_FOLDER);
